@file:Suppress("UnstableApiUsage")

package fi.jyu.intellij.comtest.rider.util

import com.intellij.openapi.project.Project
import com.intellij.platform.backend.workspace.WorkspaceModel
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rider.model.solutionAnalysisModel
import com.jetbrains.rider.projectView.solution
import com.jetbrains.rider.projectView.workspace.getFile
import com.jetbrains.rider.projectView.workspace.getSolutionEntity
import java.io.File

/**
 * Get the File reference to the currently open Solution.
 * If the solution is virtual, returns null.
 */
val Project.riderSolutionFile: File?
    get() = WorkspaceModel.getInstance(this).getSolutionEntity()?.getFile()

/**
 * Check if the project contains a Rider solution.
 */
val Project.hasRiderSolution: Boolean
    get() = this.riderSolutionFile != null

/**
 * Run the given callback when the .NET solution analysis is ready.
 * The callback is only called once.
 *
 * @param wait If true, wait for the solution analysis to be ready before running the callback.
 * @param cb The callback to run.
 */
fun Project.onSolutionAnalysisReadyFireOnce(wait: Boolean = false, cb: () -> Unit) {
    if (!wait && this.solution.solutionAnalysisModel.ready.valueOrNull == true) {
        cb()
        return
    }
    val l = this.lifetime.createNested()
    this.solution.solutionAnalysisModel.ready.change.advise(l) {
        if (it) {
            l.terminate(true)
            cb()
        }
    }
}